<?php

/**
 * @file
 * Connector to TextMaster API.
 */

/**
 * TextMaster connector class.
 */
class TmgmtTextmasterConnector {

  private $translator;

  private $serviceUrl;

  private $apiKey;

  private $apiSecret;

  /**
   * Constructor.
   *
   * @param string $service_url
   *   TextMaster API URL.
   * @param string $api_key
   *   TextMaster API key.
   * @param string $api_secret
   *   TextMaster API secret.
   * @param TMGMTTranslator $translator
   *   Translator.
   */
  public function __construct($service_url, $api_key, $api_secret, TMGMTTranslator $translator) {
    $this->serviceUrl = $service_url;
    $this->apiKey = $api_key;
    $this->apiSecret = $api_secret;
    $this->translator = $translator;
  }

  /**
   * Returns supported languages.
   *
   * @return array
   *   The list of supported languages.
   */
  public function getSupportedRemoteLanguages() {
    $supported_remote_languages = [];

    try {
      $supported_languages = $this->sendApiRequest('v1/public/languages');
      foreach ($supported_languages['languages'] as $language) {
        if (!preg_match('/[-]/', $language['code'])) {
          // Ignore all languages without '-' as these items are not supported
          // for TextMaster templates.
          continue;
        }
        $supported_remote_languages[] = $language['code'];
      }
    }
    catch (\Exception $e) {
      watchdog('tmgmt_textmaster', 'En exception occurred while trying to get Remote languages: @error', ['@error' => $e->getMessage()], WATCHDOG_ERROR);
    }
    asort($supported_remote_languages);
    return $supported_remote_languages;
  }

  /**
   * {@inheritdoc}
   */
  public function requestJobItemsTranslation(array $job_items) {
    /** @var TMGMTJob $job */
    $job = reset($job_items)->getJob();
    if ($job->isRejected()) {
      // Change the status to Unprocessed to allow submit again.
      $job->setState(TMGMT_JOB_STATE_UNPROCESSED);
    }
    try {
      $project_id = $this->createTmProject($job);
      $job->addMessage('Created a new Project in TextMaster with the id: @id', ['@id' => $project_id], 'debug');

      /** @var TMGMTJobItem $job_item */
      foreach ($job_items as $job_item) {
        $operations[] = [
          [static::class, 'createDocumentForJobItemBatchProcess'],
          [$job_item, $job, $project_id],
        ];
      }
      $batch = [
        'title' => t('Creating TextMaster Documents'),
        'operations' => $operations,
        'finished' => [static::class, 'createDocumentForJobItemBatchFinish'],
        'init_message' => t('Creating TextMaster Documents batch is starting.'),
        'progress_message' => t('Processed @current out of @total Job Items.'),
      ];

      // Set batch process for Documents creation.
      batch_set($batch);
    }
    catch (TMGMTException $e) {
      $job->rejected('Job has been rejected with following error: @error',
        ['@error' => $e->getMessage()], 'error');
    }
    return $job;
  }

  /**
   * Batch callback for Document creation process.
   *
   * @param TMGMTJobItem $job_item
   *   Job Item.
   * @param TMGMTJob $job
   *   Drupal tmgmt Job.
   * @param string $project_id
   *   ID of Project in TextMaster for this job.
   * @param array $context
   *   An array that will contain information about the
   *   status of the batch. The values in $context will retain their
   *   values as the batch progresses.
   */
  public static function createDocumentForJobItemBatchProcess(TMGMTJobItem $job_item, TMGMTJob $job, $project_id, array &$context) {
    if (empty($context['results'])) {
      // Set initial values.
      $context['results']['job_id'] = $job_item->tjid;
      $context['results']['project_id'] = $project_id;
      $context['results']['created'] = 0;
      $context['results']['errors'] = [];
    }

    try {
      /** @var TMGMTTextmasterPluginController $controller */
      $controller = $job->getTranslator()->getController();
      /** @var TmgmtTextmasterConnector $connector */
      $connector = $controller->getConnector($job->getTranslator());
      $document_id = $connector->sendFiles($job_item, $project_id);

      /** @var TMGMTRemote $remote_mapping */
      $remote_mapping = entity_create('tmgmt_remote', [
        'tjid' => $job->tjid,
        'tjiid' => $job_item->tjiid,
        'remote_identifier_1' => 'tmgmt_textmaster',
        'remote_identifier_2' => $project_id,
        'remote_identifier_3' => $document_id,
        'remote_data' => [
          'FileStateVersion' => 1,
          'TMState' => 'TranslatableSource',
          'TemplateAutoLaunch' => $connector->isTemplateAutoLaunch($job->getSetting('templates_wrapper')['project_template']),
          'WordCountFinished' => FALSE,
        ],
      ]);
      $remote_mapping->save();
      $job->addMessage('Created a new Document in TextMaster with the id: @id for Job Item: @item_label', [
        '@id' => $document_id,
        '@item_label' => $job_item->label(),
      ], 'debug');

      $context['results']['created']++;
    }
    catch (\Exception $e) {
      // Delete Remote mapping if error occurred.
      if (isset($remote_mapping)) {
        $remote_mapping->delete();
      }
      $message = t('Exception occurred while creating a Document for the job item "@job_item": @error.', [
        '@job_item' => $job_item->label(),
        '@error' => $e,
      ]);
      $job->addMessage($message, [], 'debug');

      $context['results']['errors'][] = $message;
    }

    // Inform the batch engine that we finished the operation with this item.
    $context['finished'] = 1;
  }

  /**
   * Batch 'finished' callback for Creating TextMaster documents process.
   *
   * @param bool $success
   *   Batch success.
   * @param array $results
   *   Results.
   * @param array $operations
   *   Operations.
   */
  public static function createDocumentForJobItemBatchFinish($success, array $results, array $operations) {
    if (!$success) {
      return;
    }
    $errors = $results['errors'];
    $created = $results['created'];
    $job = tmgmt_job_load($results['job_id']);

    if (count($errors) == 0 && !empty($created)) {

      // Set Job state.
      if (!$job->isRejected()) {
        $mappings = $job->getRemoteMappings();
        $job_remote_data = end($mappings);
        $auto_launch = $job_remote_data->remote_data['TemplateAutoLaunch'];
        if ($auto_launch) {
          $job->submitted();
        }
        else {
          $job->setState(TMGMT_JOB_STATE_UNPROCESSED, 'The translation job has been submitted.');
        }
      }
      drupal_set_message(t('@created document(s) was(were) created in TextMaster for Job "@job_label".', [
        '@created' => $created,
        '@job_label' => $job->label(),
      ]));
      drupal_goto('admin/tmgmt/overview');
    }
    // Some errors occurred. Show them.
    elseif (!empty($created)) {
      $message = t('Project for job @job_label was not finalized. @created documents were created in TextMaster. @errors_count error(s) occurred during Document creation: @error', [
        '@job_label' => $job->label(),
        '@created' => $created,
        '@errors_count' => count($errors),
        '@error' => implode('; ', $errors),
      ]);
      drupal_set_message($message);
    }
    else {
      $message = t('Project for job @job_label was not finalized. Error(s) occurred during Document creation: @error', [
        '@job_label' => $job->label(),
        '@error' => implode('; ', $errors),
      ]);
      drupal_set_message($message);
    }
  }

  /**
   * Checks the TextMaster account.
   *
   * @return bool
   *   A success or failure.
   */
  public function checkTextmasterAuthentication() {
    try {
      $result = $this->getTmAccountInfo();
      if ($result) {
        // Successfully Authenticated.
        return TRUE;
      }
    }
    catch (TMGMTException $ex) {
      watchdog('tmgmt_textmaster', 'Unable to log in to TextMaster API: @error', ['@error' => $ex->getMessage()], WATCHDOG_WARNING);
    }
    return FALSE;
  }

  /**
   * Gets the TextMaster Account information.
   *
   * @return array|int|null
   *   Account info.
   */
  public function getTmAccountInfo() {
    return $this->sendApiRequest('v1/clients/users/me');
  }

  /**
   * Get TextMaster API templates.
   *
   * @return array|int|null|false
   *   Result of the API request or FALSE.
   */
  public function getTmApiTemplates() {

    $cache = cache_get('tmgmt_textmaster_api_templates');
    if (!empty($cache)) {
      return $cache->data;
    }

    try {
      $templates = $this->allPagesResult('v1/clients/api_templates', 'api_templates');
      cache_set('tmgmt_textmaster_api_templates', $templates);
      return $templates;
    }
    catch (TMGMTException $e) {
      watchdog('tmgmt_textmaster',
        'Could not get TextMaster API templates: @error',
        ['@error' => $e->getMessage()],
        WATCHDOG_ERROR);
    }
    return FALSE;
  }

  /**
   * Function to check if api template "auto_launch" parameter is set TRUE.
   *
   * @param string $api_template_id
   *   The ID of TextMaster API template.
   *
   * @return bool
   *   True if template auto_launch parameter is set to true.
   */
  public function isTemplateAutoLaunch($api_template_id) {
    $templates = $this->getTmApiTemplates();
    foreach ($templates as $template) {
      if ($template['id'] === $api_template_id) {
        return $template['auto_launch'];
      }
    }
    return FALSE;
  }

  /**
   * Function to get all pages result.
   *
   * @param string $request_path
   *   Path for request.
   * @param string $result_key
   *   The array key for results.
   * @param array $previous_pages_result
   *   The array with previous pages values.
   *
   * @return array|int|null
   *   Result of the API request.
   */
  public function allPagesResult($request_path, $result_key, array $previous_pages_result = []) {
    $result = $this->sendApiRequest($request_path);
    $all_pages_list = array_merge($result[$result_key], $previous_pages_result);
    if (isset($result['next_page'])) {
      // Remove basic API path here to avoid duplication.
      $base_path = parse_url($request_path, PHP_URL_PATH);
      $next_page_query = parse_url($result['next_page'], PHP_URL_QUERY);
      $next_page_path = $base_path . '?' . $next_page_query;
      return $this->allPagesResult($next_page_path, $result_key, $all_pages_list);
    }
    return $all_pages_list;
  }

  /**
   * Sends a request to the TextMaster API.
   *
   * @param string $path
   *   API path.
   * @param string $method
   *   (Optional) HTTP method.
   * @param array $params
   *   (Optional) API params.
   * @param bool $code
   *   (Optional) If true, return only the response HTTP status code.
   * @param bool $download
   *   (Optional) If true, return the response body as a downloaded content.
   * @param string $body
   *   (Optional) An optional request body.
   *
   * @return array|int|null
   *   Result of the API request.
   *
   * @throws TMGMTException
   */
  public function sendApiRequest($path, $method = 'GET', array $params = [], $code = FALSE, $download = FALSE, $body = NULL) {
    $result = NULL;
    try {
      $result = $this->request($path, $method, $params, $code, $download, $body);
    }
    catch (TMGMTException $ex) {
      if ($ex->getCode() == 401) {
        // Authentication failed , try to re-connect.
        $result = $this->request($path, $method, $params, $code, $download, $body);
      }
      else {
        throw $ex;
      }
    }
    return $result;
  }

  /**
   * Does a request to TextMaster API.
   *
   * @param string $path
   *   Resource path.
   * @param string $method
   *   (Optional) HTTP method (GET, POST...). By default uses GET method.
   * @param array $params
   *   (Optional) Form parameters to send to TextMaster API.
   * @param bool $code
   *   (Optional) If we want to return the status code of the call. FALSE by
   *   default.
   * @param bool $download
   *   (Optional) If we expect resource to be downloaded. FALSE by default.
   * @param string $body
   *   (Optional) Body of the POST request. NULL by
   *   default.
   *
   * @return array|int|null
   *   Response array or status code.
   *
   * @throws TMGMTException
   */
  public function request($path, $method = 'GET', array $params = [], $code = FALSE, $download = FALSE, $body = NULL) {
    $options = [];
    if (empty($this->serviceUrl)) {
      watchdog('tmgmt_textmaster', 'Attempt to call TextMaster API when service_url is not set: @path', ['@path' => $path], WATCHDOG_WARNING);
      return [];
    }
    $url = $this->serviceUrl . '/' . $path;

    if ($body) {
      $options['data'] = $body;
    }
    else {
      if ($method == 'GET') {
        $url .= '?' . http_build_query($params);
      }
      else {
        $options['data'] = json_encode($params);
        $options['method'] = $method;
      }
    }

    // Default headers for TextMaster Api requests.
    $date = $this->utcDate();
    $options['headers'] = [
      'Apikey' => $this->apiKey,
      'Date' => $date,
      'Signature' => $this->getTextmasterSignature($date, $this->apiSecret),
      'Content-Type' => 'application/json',
      // Enhancement #309335: track from which platform the project was created.
      'HTTP_X_PARTNER_ID' => 'ad40859b-5b20-458b-a3fe-73e93fa34ea1',
    ];

    $response = drupal_http_request($url, $options);
    $received_data = $response->data;

    if ($response->code != 200) {
      watchdog('tmgmt_textmaster', '@method Request to @url:<br>
        <ul>
            <li>Request: @request</li>
            <li>Response: @response</li>
        </ul>
        ', [
          '@method' => $method,
          '@url' => $url,
          '@request' => json_encode($options),
          '@response' => $received_data,
        ], WATCHDOG_ERROR);
      if ($code) {
        return $response->code;
      }
      throw new TMGMTException('Unable to connect to TextMaster service due to following error: @error at @url',
        ['@error' => $received_data, '@url' => $url], $response->code);
    }

    watchdog('tmgmt_textmaster', '@method Request to @url:<br>
        <ul>
            <li>Request: @request</li>
            <li>Response: @response</li>
        </ul>
        ', [
          '@method' => $method,
          '@url' => $url,
          '@request' => json_encode($options),
          '@response' => $received_data,
        ], WATCHDOG_DEBUG);

    if ($code) {
      return $response->code;
    }

    // If we are expecting a download, just return received data.
    if ($download) {
      return $received_data;
    }
    $received_data = json_decode($received_data, TRUE);

    return $received_data;
  }

  /**
   * Creates new translation project at TextMaster.
   *
   * @param TMGMTJob $job
   *   The job.
   *
   * @return int
   *   TextMaster Project id.
   *
   * @throws TMGMTException
   */
  public function createTmProject(TMGMTJob $job) {
    // Prepare parameters for Project API.
    $name = $job->label() ?: 'Drupal TMGMT project ' . $job->tjid;
    $callback_url = url('tmgmt_textmaster_project_finalized_callback', ['absolute' => TRUE]);
    $params = [
      'project' => [
        'name' => $name,
        'activity_name' => 'translation',
        'api_template_id' => $job->getSetting('templates_wrapper')['project_template'],
        'category' => 'C033',
        'callback' => [
          'project_finalized' => [
            "url" => $callback_url,
            "format" => "json",
          ],
        ],
      ],
    ];
    $result = $this->sendApiRequest('v1/clients/projects', 'POST', $params);

    return $result['id'];
  }

  /**
   * Finalizes TextMaster project.
   *
   * @param string $project_id
   *   TextMaster project id.
   * @param \TMGMTJob $job
   *   TMGMT Job.
   *
   * @return array|int|null|false
   *   Result of the API request or FALSE.
   */
  public function finalizeTmProject($project_id, TMGMTJob $job) {
    try {
      $result = $this->sendApiRequest('v1/clients/projects/' . $project_id . '/finalize', 'PUT', []);
      $job->addMessage('Finalization request was sent for TextMaster Project with the id: @id', [
        '@id' => $project_id,
      ], 'debug');
      return $result;
    }
    catch (TMGMTException $e) {
      watchdog('tmgmt_textmaster', 'Could not finalize TextMaster Project with the id: @id ( @error )', [
        '@id' => $project_id,
        '@error' => $e->getMessage(),
      ], WATCHDOG_ERROR);
    }
    return FALSE;
  }

  /**
   * Launches TextMaster project.
   *
   * @param string $project_id
   *   TextMaster project id.
   *
   * @return array|int|null|false
   *   Result of the API request or FALSE.
   */
  public function launchTmProject($project_id) {
    try {
      $result = $this->sendApiRequest('v1/clients/projects/' . $project_id . '/launch', 'PUT', []);
      return $result;
    }
    catch (TMGMTException $e) {
      watchdog('tmgmt_textmaster', 'Could not launch TextMaster Project: @error', ['@error' => $e->getMessage()], WATCHDOG_ERROR);
    }
    return FALSE;
  }

  /**
   * Get TextMaster project.
   *
   * @param string $project_id
   *   TextMaster project id.
   *
   * @return array|int|null|false
   *   Result of the API request or FALSE.
   */
  public function getTmProject($project_id) {
    try {
      return $this->sendApiRequest('v1/clients/projects/' . $project_id);
    }
    catch (TMGMTException $e) {
      watchdog('tmgmt_textmaster', 'Could not get TextMaster Project: @error', ['@error' => $e->getMessage()], WATCHDOG_ERROR);
    }
    return FALSE;
  }

  /**
   * Pause TextMaster project.
   *
   * @param string $project_id
   *   TextMaster project id.
   *
   * @return array|int|null|false
   *   Result of the API request or FALSE.
   */
  public function pauseTmProject($project_id) {
    try {
      $result = $this->sendApiRequest('v1/clients/projects/' . $project_id . '/pause', 'PUT', []);
      return $result;
    }
    catch (TMGMTException $e) {
      watchdog('tmgmt_textmaster', 'Could not pause TextMaster Project: @error', ['@error' => $e->getMessage()], WATCHDOG_ERROR);
    }
    return FALSE;
  }

  /**
   * Cancel TextMaster project.
   *
   * @param string $project_id
   *   TextMaster project id.
   *
   * @return array|int|null|false
   *   Result of the API request or FALSE.
   */
  public function cancelTmProject($project_id) {
    try {
      $result = $this->sendApiRequest('v1/clients/projects/' . $project_id . '/cancel', 'PUT', []);
      return $result;
    }
    catch (TMGMTException $e) {
      watchdog('tmgmt_textmaster', 'Could not cancel TextMaster Project: @error', ['@error' => $e->getMessage()], WATCHDOG_ERROR);
    }
    return FALSE;
  }

  /**
   * Send files to TextMaster.
   *
   * @param TMGMTJobItem $job_item
   *   The Job.
   * @param int $project_id
   *   TextMaster project id.
   *
   * @return int
   *   TextMaster Document Id.
   */
  public function sendFiles(TMGMTJobItem $job_item, $project_id) {
    // Prepare parameters for Job API.
    $xliff_converter = tmgmt_file_format_controller('xlf');

    $job_item_id = $job_item->tjiid;
    $target_language = $job_item->getJob()->target_language;
    $conditions = ['tjiid' => ['value' => $job_item_id]];
    $xliff = $xliff_converter->export($job_item->getJob(), $conditions);
    $name = "JobID_{$job_item->tjid}_JobItemID_{$job_item_id}_{$job_item->getJob()->source_language}_{$target_language}";

    $remote_file_url = $this->createTmRemoteFile($xliff, $name);
    $document_id = $this->createTmDocument($project_id, $remote_file_url, $name);

    return $document_id;
  }

  /**
   * Creates a file resource at TextMaster.
   *
   * @param string $xliff
   *   .XLIFF string to be translated. It is send as a file.
   * @param string $name
   *   File name of the .XLIFF file without extension.
   *
   * @return string
   *   The URL of uploaded file.
   *
   * @throws TMGMTException
   */
  public function createTmRemoteFile($xliff, $name) {
    $file_name = $name . '.xliff';
    $file_hash = hash('sha256', $xliff);

    // Set Parametres to request for upload properties from TextMaster API.
    $params = [
      'file_name' => $file_name,
      'hashed_payload' => $file_hash,
    ];
    $upload_properties = $this->sendApiRequest('v1/clients/s3_upload_properties.json', 'POST', $params);
    if (!isset($upload_properties['url']) || !isset($upload_properties['headers'])) {
      throw new TMGMTException('Could not obtain upload properties from TextMaster API');
    }
    // Set method, headers and body for request.
    $options['method'] = 'PUT';
    $options['headers'] = $upload_properties['headers'];
    $options['headers']['Content-Type'] = 'application/xml';
    $options['data'] = $xliff;
    // We don't need apiRequest here just common http request.
    $file_response = drupal_http_request($upload_properties['url'], $options);
    if ($file_response->code != 200) {
      throw new TMGMTException('Could not Upload the file ' . $file_name . ' to TextMaster.');
    }

    return $upload_properties['url'];
  }

  /**
   * Get TextMaster document.
   *
   * @param string $project_id
   *   TextMaster project id.
   * @param string $document_id
   *   TextMaster document id.
   *
   * @return array|int|null
   *   Result of the API request or empty array.
   */
  public function getTmDocument($project_id, $document_id) {
    try {
      return $this->sendApiRequest('v1/clients/projects/' . $project_id . '/documents/' . $document_id, 'GET');
    }
    catch (TMGMTException $e) {
      watchdog('tmgmt_textmaster', 'Could not get the TextMaster Document: @error', ['@error' => $e->getMessage()], WATCHDOG_ERROR);
    }
    return [];
  }

  /**
   * Creates a new job at TextMaster.
   *
   * @param string $project_id
   *   Project ID.
   * @param string $remote_file_url
   *   Remote file url.
   * @param string $document_title
   *   Remote Document title.
   *
   * @return string
   *   TextMaster Document ID.
   */
  public function createTmDocument($project_id, $remote_file_url, $document_title) {
    $in_review_url = url('tmgmt_textmaster_in_review_callback', ['absolute' => TRUE]);
    $word_count_url = url('tmgmt_textmaster_word_count_finished_callback', ['absolute' => TRUE]);
    $params = [
      'document' => [
        'title' => $document_title,
        'remote_file_url' => $remote_file_url,
        'deliver_work_as_file' => 'true',
        'perform_word_count' => 'true',
        'callback' => [
          'in_review' => [
            "url" => $in_review_url,
            "format" => "json",
          ],
          'word_count_finished' => [
            "url" => $word_count_url,
            "format" => "json",
          ],
        ],
      ],
    ];
    $result = $this->sendApiRequest('v1/clients/projects/' . $project_id . '/documents', 'POST', $params);

    return $result['id'];
  }

  /**
   * Complete TextMaster document.
   *
   * @param string $project_id
   *   TextMaster project id.
   * @param string $document_id
   *   TextMaster document id.
   *
   * @return array|int|null
   *   Result of the API request or empty array.
   */
  public function completeTmDocument($project_id, $document_id) {
    $params = [
      'satisfaction' => 'positive',
      'message' => 'Well done!',
    ];
    try {
      return $this->sendApiRequest('v1/clients/projects/' . $project_id . '/documents/' . $document_id . '/complete', 'PUT', $params);
    }
    catch (TMGMTException $e) {
      watchdog('tmgmt_textmaster', 'Could not complete the TextMaster Document: @error', ['@error' => $e->getMessage()], WATCHDOG_ERROR);
    }
    return [];
  }

  /**
   * Ask for document revision in TextMaster.
   *
   * @param string $project_id
   *   TextMaster project id.
   * @param string $document_id
   *   TextMaster document id.
   * @param string $message
   *   Revision message.
   *
   * @return array|int|null|false
   *   Result of the API request or FALSE.
   */
  public function createTmSupportMessage($project_id, $document_id, string $message) {
    $params = [
      'support_message' => [
        'message' => $message,
      ],
    ];
    try {
      return $this->sendApiRequest('v1/clients/projects/' . $project_id . '/documents/' . $document_id . '/support_messages', 'POST', $params);
    }
    catch (TMGMTException $e) {
      watchdog('tmgmt_textmaster', 'Could not create TextMaster support message: @error', ['@error' => $e->getMessage()], WATCHDOG_ERROR);

      \Drupal::logger('tmgmt_textmaster')
        ->error('Could not complete the TextMaster Document: @error', ['@error' => $e->getMessage()]);
    }
    return [];
  }

  /**
   * Parses received translation from TextMaster and returns unflatted data.
   *
   * @param string $data
   *   Base64 encode data, received from TextMaster.
   *
   * @return array
   *   Unflatted data.
   */
  protected function parseTranslationData($data) {
    $xlf_converter = tmgmt_file_format_controller('xlf');
    // Import given data using XLIFF converter. Specify that passed content is
    // not a file.
    return $xlf_converter->import($data, FALSE);
  }

  /**
   * Fetches translations for job items of a given job.
   *
   * @param TMGMTJob $job
   *   A job containing job items that translations will be fetched for.
   */
  public function fetchTranslatedFiles(TMGMTJob $job) {
    $job_items = $job->getItems();
    $operations = [];
    foreach ($job_items as $job_item) {
      $operations[] = [
        [static::class, 'fetchTranslationsBatchProcess'],
        [$job_item],
      ];
    }
    $batch = [
      'title' => t('Pulling translations'),
      'operations' => $operations,
      'finished' => [static::class , 'fetchTranslationsBatchFinish'],
      'init_message' => t('Pull Translation batch is starting.'),
      'progress_message' => t('Processed @current out of @total Job Items.'),
    ];
    batch_set($batch);
  }

  /**
   * Batch callback for pull Job translations process.
   *
   * @param TMGMTJobItem $job_item
   *   Job Item.
   * @param array $context
   *   An array that will contain information about the
   *   status of the batch. The values in $context will retain their
   *   values as the batch progresses.
   */
  public static function fetchTranslationsBatchProcess(TMGMTJobItem $job_item, array &$context) {
    if (empty($context['results'])) {
      // Set initial values.
      $context['results']['job_id'] = $job_item->tjid;
      $context['results']['errors'] = [];
      $context['results']['translated'] = 0;
    }

    $translated = $context['results']['translated'];
    $errors = $context['results']['errors'];
    $job = $job_item->getJob();

    /** @var TMGMTTextmasterPluginController $controller */
    $controller = $job->getTranslator()->getController();
    /** @var TmgmtTextmasterConnector $connector */
    $connector = $controller->getConnector($job->getTranslator());
    $is_item_translated = FALSE;
    // Get all existing mappings for this Job Item.
    $mappings = $job_item->getRemoteMappings();
    /** @var TMGMTRemote $mapping */
    foreach ($mappings as $mapping) {
      // Prepare parameters for TextMaster API.
      $document_id = $mapping->remote_identifier_3;
      $project_id = $mapping->remote_identifier_2;
      $info = $connector->getTmDocument($project_id, $document_id);
      if (empty($info)) {
        $job->addMessage('Error fetching the job item: @job_item. TextMaster document @document_id not found.',
          [
            '@job_item' => $job_item->label(),
            '@document_id' => $document_id,
          ], 'error');
        $errors[] = 'TextMaster document ' . $document_id . ' not found, it was probably deleted.';
      }

      if (!array_key_exists('status', $info)
        || !$connector->isRemoteTranslationCompleted($info['status'])
      ) {
        continue;
      }

      try {
        $connector->addTranslationToJob($job, $info['status'], $project_id, $document_id, $info['author_work']);
        $is_item_translated = TRUE;
      }
      catch (TMGMTException $e) {
        $job->addMessage('Exception occurred while fetching the job item "@job_item": @error.', [
          '@job_item' => $job_item->label(),
          '@error' => $e,
        ], 'error');
        $errors[] = 'Exception occurred while fetching the job item ' . $job_item->label();
      }
    }
    if ($is_item_translated) {
      $translated++;
    }

    // Set  results:
    $context['results']['translated'] = $translated;
    $context['results']['untranslated'] = count($job->getItems()) - $translated;
    $context['results']['errors'] = $errors;

    // Inform the batch engine that we finished the operation with this item.
    $context['finished'] = 1;
  }

  /**
   * Batch 'finished' callback for pull Job translations process.
   *
   * @param bool $success
   *   Batch success.
   * @param array $results
   *   Results.
   * @param array $operations
   *   Operations.
   */
  public static function fetchTranslationsBatchFinish($success, array $results, array $operations) {
    if (!$success) {
      return;
    }
    $translated = $results['translated'];
    $untranslated = $results['untranslated'];
    $errors = $results['errors'];
    $job = tmgmt_job_load($results['job_id']);
    if (count($errors) == 0) {
      if ($untranslated == 0 && $translated != 0) {
        $job->addMessage(t('Fetched translations for @translated job item(s).', ['@translated' => $translated]));
      }
      elseif ($translated == 0) {
        drupal_set_message(t('No job item has been translated yet.'));
      }
      else {
        $job->addMessage(t('Fetched translations for @translated job item(s), @untranslated is(are) not translated yet.', [
          '@translated' => $translated,
          '@untranslated' => $untranslated,
        ]));
      }
    }
    else {
      drupal_set_message(t('Error(s) occurred during fetching translations for Job: @error', ['@error' => implode('; ', $errors)]));
    }

    tmgmt_ui_write_request_messages($job);
  }

  /**
   * Checks if the translation has one of the completed statuses.
   *
   * @param string $status
   *   Status code.
   *
   * @return bool
   *   True if completed.
   */
  public function isRemoteTranslationCompleted($status) {
    return $status == 'in_review' || $status == 'completed';
  }

  /**
   * Retrieve the data of a file in a state.
   *
   * @param TMGMTJob $job
   *   The Job to which will be added the data.
   * @param string $document_state
   *   The state of the file.
   * @param int $project_id
   *   The project ID.
   * @param string $document_id
   *   The Document ID.
   * @param string $remote_file_url
   *   Translated file url.
   *
   * @throws TMGMTException
   */
  public function addTranslationToJob(TMGMTJob $job, $document_state, $project_id, $document_id, $remote_file_url) {
    $translated_file_response = drupal_http_request($remote_file_url);
    $translated_file_content = $translated_file_response->data;
    $file_data = $this->parseTranslationData($translated_file_content);
    if ($this->isRemoteTranslationCompleted($document_state)) {
      $status = TMGMT_DATA_ITEM_STATE_TRANSLATED;
    }
    else {
      $status = TMGMT_DATA_ITEM_STATE_PENDING;
    }
    $job->addTranslatedData($file_data, []);
    /** @var TMGMTRemoteController $remote_mapping_controller */
    $remote_mapping_controller = entity_get_controller('tmgmt_remote');
    $mappings = $remote_mapping_controller->loadByRemoteIdentifier('tmgmt_textmaster', $project_id, $document_id);
    /** @var TMGMTRemote $mapping */
    $mapping = end($mappings);
    $mapping->removeRemoteData('TMState');
    $mapping->addRemoteData('TMState', $status);
    $mapping->save();
  }

  /**
   * Generates TextMaster Api signature. See https://www.app.textmaster.com/api-documentation#authentication-signature-creation.
   *
   * @param string $date
   *   Date gmt/utc in format 'Y-m-d H:i:s'.
   * @param string $api_secret
   *   TextMaster Api Secret.
   *
   * @return string
   *   TextMaster Api signature.
   */
  public function getTextmasterSignature($date, $api_secret) {
    $signature = sha1($api_secret . $date);
    return $signature;
  }

  /**
   * Get gmt/utc date in format 'Y-m-d H:i:s'.
   *
   * @return string
   *   Date gmt/utc in format 'Y-m-d H:i:s'.
   */
  public function utcDate() {
    return gmdate('Y-m-d H:i:s');
  }

}
