<?php

/**
 * @file
 * Provides TextMaster translation plugin controller.
 */

/**
 * TextMaster translator controller class.
 */
class TMGMTTextmasterTranslatorUIController extends TMGMTDefaultTranslatorUIController {

  const TEXTMASTER_API_URL = 'http://api.textmaster.com';

  /**
   * {@inheritdoc}
   */
  public function pluginSettingsForm($form, &$form_state, TMGMTTranslator $translator, $busy = FALSE) {
    $tm_app_url = $this->getApplicationUrl($translator);
    $tm_api_key_url = url($tm_app_url . '/clients/api_info');

    $form['textmaster_service_url'] = [
      '#type' => 'textfield',
      '#title' => t('TextMaster API url'),
      '#default_value' => $translator->getSetting('textmaster_service_url') ?: static::TEXTMASTER_API_URL,
      '#description' => t('Please enter the TextMaster API base url.'),
      '#required' => TRUE,
    ];
    $form['textmaster_api_key'] = [
      '#type' => 'textfield',
      '#title' => t('TextMaster API key'),
      '#default_value' => $translator->getSetting('textmaster_api_key') ?: '',
      '#description' => t('Please enter the TextMaster API key. You can find it <a href="@api_key_url" target="_blank">here</a>', [
        '@api_key_url' => $tm_api_key_url,
      ]),
      '#required' => TRUE,
    ];
    $form['textmaster_api_secret'] = [
      '#type' => 'textfield',
      '#title' => t('TextMaster API secret'),
      '#default_value' => $translator->getSetting('textmaster_api_secret') ?: '',
      '#description' => t('Please enter your TextMaster API secret. You can find it <a href="@api_key_url" target="_blank">here</a>', [
        '@api_key_url' => $tm_api_key_url,
      ]),
      '#required' => TRUE,
    ];
    return parent::pluginSettingsForm($form, $form_state, $translator, $busy);
  }

  /**
   * {@inheritdoc}
   */
  public function checkoutInfo(TMGMTJob $job) {
    $form = [];
    if ($job->isActive()) {
      $form['actions']['pull'] = [
        '#type' => 'submit',
        '#value' => t('Pull translations'),
        '#submit' => [[$this, 'submitPullTranslations']],
        '#weight' => -10,
      ];
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function checkoutSettingsForm($form, &$form_state, TMGMTJob $job) {

    /** @var TMGMTTranslator $translator */
    $translator = $job->getTranslator();
    /** @var TMGMTTextmasterPluginController $controller */
    $controller = $translator->getController();
    /** @var TmgmtTextmasterConnector $connector */
    $connector = $controller->getConnector($translator);
    $tm_app_url = $this->getApplicationUrl($translator);

    // Account Credits.
    $account_info = $connector->getTmAccountInfo();
    if (!empty($account_info['wallet'])) {
      $buy_credits_url = url($tm_app_url . '/clients/payment_requests/new');
      $settings['account_credits'] = [
        '#type' => 'item',
        '#title' => t('Available credits: @current_money @currency_code', [
          '@current_money' => $account_info['wallet']['current_money'],
          '@currency_code' => $account_info['wallet']['currency_code'],
        ]),
        '#markup' => t('<a href="@buy_credits_url" target="_blank">Buy credits on TextMaster</a>', [
          '@buy_credits_url' => $buy_credits_url,
        ]),
      ];
    }

    // Project Price.
    $settings['project_price'] = [
      '#access' => FALSE,
      '#type' => 'textfield',
      '#title' => t('Project Price'),
      '#description' => t('TextMaster Project price.'),
      '#default_value' => $job->getSetting('project_price'),
    ];

    // Project Templates.
    $settings['templates_wrapper'] = [
      '#type' => 'container',
      '#attributes' => ['id' => 'templates-wrapper'],
    ];
    // Update templates button.
    $settings['update_template_list'] = [
      '#type' => 'button',
      '#value' => t('Update templates'),
      '#description' => t('If you added new template in TextMaster click on this button to update the list in Drupal.'),
      '#validate' => [[$this, 'updateTemplatesValidate']],
      '#ajax' => [
        'callback' => [$this, 'updateTemplatesSelectlist'],
        'wrapper' => 'templates-wrapper',
        'method' => 'replace',
      ],
      '#limit_validation_errors' => [],
      '#weight' => 10,
    ];
    // Project templates list.
    if ($form_state['rebuild']
      && $form_state['triggering_element']['#value'] == $settings['update_template_list']['#value']
    ) {
      // Its ajax response to "Update templates".Invalidate templates cache.
      cache_clear_all('tmgmt_textmaster_api_templates', 'cache');
    }
    $templates = $this->getTemplatesList($controller, $job);
    $settings['templates_wrapper']['project_template'] = [
      '#type' => 'select',
      '#title' => t('Project template'),
      '#options' => $templates,
      '#description' => t('Select a TextMaster project template.'),
      '#required' => TRUE,
      '#default_value' => $job->getSetting('templates_wrapper')['project_template'],
    ];
    // Add template link.
    $settings['templates_wrapper']['add_template'] = [
      '#type' => 'item',
      '#title' => t('Want to add project template?'),
      '#markup' => t('You can create it <a href="@template_url" target="_blank">here</a>', [
        '@template_url' => url($tm_app_url . '/clients/project_templates/api_templates'),
      ]),
    ];

    // Add message about word count.
    if ($form_state['rebuild']
      && $form_state['triggering_element']['#value'] == 'textmaster') {
      drupal_set_message(t('Please note that Drupal word count may differ from TextMaster.'), 'warning');
    }
    return $settings;
  }

  /**
   * Submit callback to pull translations form TextMaster.
   */
  public function submitPullTranslations(array $form, $form_state) {
    /** @var TMGMTJob $job */
    $job = $form_state['tmgmt_job'];

    /** @var TMGMTTextmasterPluginController $controller */
    $controller = $job->getTranslator()->getController();
    /** @var TmgmtTextmasterConnector $connector */
    $connector = $controller->getConnector($job->getTranslator());
    $connector->fetchTranslatedFiles($job);
  }

  /**
   * {@inheritdoc}
   */
  public function reviewFormValidate($form, &$form_state, TMGMTJobItem $item) {
    if ($item->getTranslator()->plugin != 'textmaster') {
      return;
    }
    if ($form_state['triggering_element']['#value'] == $form['actions']['save']['#value']) {
      // Allow 'Save' action for job item.
      return;
    }
    /** @var TMGMTTextmasterPluginController $controller */
    $controller = $item->getTranslatorController();
    /** @var TmgmtTextmasterConnector $connector */
    $connector = $controller->getConnector($item->getTranslator());
    $remotes = $item->getRemoteMappings();
    /** @var TMGMTRemote $remote */
    $remote = end($remotes);
    $project_id = $remote->remote_identifier_2;
    $document_id = $remote->remote_identifier_3;
    $tm_document_data = $connector->getTmDocument($project_id, $document_id);
    if (empty($tm_document_data)) {
      form_set_error('accept', t('Could not get the TextMaster Document "@document_id" to complete it.', ['@document_id' => $document_id]));
    }
    if (!array_key_exists('status', $tm_document_data)
      || !$connector->isRemoteTranslationCompleted($tm_document_data['status'])
    ) {
      form_set_error('accept', t('The translation for this job item can not be accepted as the TextMaster document "@document_id" status is "@status".', [
        '@document_id' => $document_id,
        '@status' => $tm_document_data['status'],
      ]));
      return FALSE;
    }
    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function reviewFormSubmit($form, &$form_state, TMGMTJobItem $item) {
    if ($item->getTranslator()->plugin != 'textmaster') {
      return;
    }
    // Check if the user clicked on 'Save as completed'.
    if (empty($form['actions']['accept']) || $form_state['triggering_element']['#value'] != $form['actions']['accept']['#value']) {
      return;
    }

    // Get TextMaster Document ID and Project ID.
    $remotes = $item->getRemoteMappings();
    /** @var TMGMTRemote $remote */
    $remote = end($remotes);
    $project_id = $remote->remote_identifier_2;
    $document_id = $remote->remote_identifier_3;
    /** @var TMGMTTextmasterPluginController $controller */
    $controller = $item->getTranslatorController();
    /** @var TmgmtTextmasterConnector $connector */
    $connector = $controller->getConnector($item->getTranslator());
    $tm_document_data = $connector->getTmDocument($project_id, $document_id);

    // Check document status. Only "in_review" documents can be completed.
    if (!_is_textmaster_document_status($tm_document_data, 'in_review')) {

      // This Document must be already completed as Job item passed validation.
      $message = t('Could not complete TextMaster document "@document_id" with status "@status"', [
        '@document_id' => $document_id,
        '@status' => $tm_document_data['status'],
      ]);
      drupal_set_message($message);
      $item->addMessage('Could not complete TextMaster document "@document_id" with status "@status"', [
        '@document_id' => $document_id,
        '@status' => $tm_document_data['status'],
      ]);

      return;
    }

    // Complete document in TextMaster.
    $result = $connector->completeTmDocument($project_id, $document_id);

    // Show the result messages.
    if (!empty($result)) {

      // Success.
      $item->addMessage('TextMaster Document "@document_id" was completed', [
        '@document_id' => $document_id,
      ]);
    }
    else {

      // Inform about failure.
      $item->addMessage('Could not complete TextMaster Document "@document_id"', [
        '@document_id' => $document_id,
      ], 'error');
      drupal_set_message(t('Could not complete TextMaster Document "@document_id"', [
        '@document_id' => $document_id,
      ]), 'error');
    }

  }

  /**
   * Set a value in form_state to rebuild the form and fill it with data.
   *
   * @param array $form
   *   Form.
   * @param array $form_state
   *   FormState.
   */
  public function updateTemplatesValidate(array &$form, array $form_state) {
    $form_state['rebuild'] = TRUE;
  }

  /**
   * Ajax callback to update TextMaster templates list.
   *
   * @param array $form
   *   Form.
   * @param array $form_state
   *   FormState.
   *
   * @return array
   *   Array for ajax response.
   */
  public function updateTemplatesSelectlist(array $form, array $form_state) {
    return $form['translator_wrapper']['settings']['templates_wrapper'];
  }

  /**
   * Get TextMaster templates list filtered by Job source and target language.
   *
   * @param TMGMTTextmasterPluginController $controller
   *   TextMaster plugin controller.
   * @param TMGMTJob $job
   *   TMGMT Job entity.
   *
   * @return array
   *   Filtered array of templates.
   */
  public function getTemplatesList(TMGMTTextmasterPluginController $controller, TMGMTJob $job) {
    /** @var TmgmtTextmasterConnector $connector */
    $connector = $controller->getConnector($job->getTranslator());
    $templates_json = $connector->getTmApiTemplates();
    $sourceLang = $controller->mapToRemoteLanguage($job->getTranslator(), $job->source_language);
    $targetLang = $controller->mapToRemoteLanguage($job->getTranslator(), $job->target_language);
    $templates = [];
    foreach ($templates_json as $template) {
      // Display only templates which match the selected source & target langs.
      if ($template['language_from'] === $sourceLang && $targetLang === $template['language_to']) {
        $templates[$template['id']] = $template['name'];
      }
    }
    return $templates;
  }

  /**
   * Get TextMaster Application URL.
   *
   * @param TMGMTTranslator $translator
   *   TMGMT Translator.
   *
   * @return string
   *   TextMaster Application URL.
   */
  public static function getApplicationUrl(TMGMTTranslator $translator) {
    $service_url = $translator->getSetting('textmaster_service_url');
    if (!isset($service_url) || !is_string($service_url)) {
      $service_url = '';
    }
    if (strpos($service_url, 'sandbox') !== FALSE) {

      return 'https://www.app.sandbox.textmaster.com';
    }

    return 'https://www.app.textmaster.com';
  }

}
